import json
import os
from unittest import mock

import pytest

from ammcpc.ammcpc import MediaConchPolicyCheckerCommand

HERE = os.path.dirname(os.path.realpath(__file__))
# For the purposes of the fails.mkv and passes.mkv files, both policy files
# behave the same. At some point MediaConch switched over to the more readable
# .xml policy file format but retaned backwards-compatible support for the .xsl
# format.
POLICY_XSL_NAME = "NYULib_MKVFFV1_MODIFIED.xsl"
POLICY_XSL_PATH = os.path.join(HERE, "policies", POLICY_XSL_NAME)
POLICY_XML_NAME = "NYULib_MKVFFV1_MODIFIED.xml"
POLICY_XML_PATH = os.path.join(HERE, "policies", POLICY_XML_NAME)
FILE_FAILS_PATH = os.path.join(HERE, "files", "fails.mkv")
FILE_PASSES_PATH = os.path.join(HERE, "files", "passes.mkv")

# These fixtures (policies and MKV files) validate as expected up until
# mediaconch 18.03-2, but they fail in newer versions. These variables mock the
# expected output so the tests do not depend on mediaconch being installed.
XSL_FAILED_CHECK_OUTPUT = """
<MediaConch xmlns="https://mediaarea.net/mediaconch" xmlns:mi="https://mediaarea.net/mediainfo" version="0.1">
  <media ref="/fails.mkv">
    <policyChecks>
      <name>Preservation Master File Recommendations - Matroska/FFV1 (NYU Libraries)</name>
      <description/>
      <check name="General Format equals Matroska">
        <context field="Format" value="Matroska"/>
        <test tracktype="General" actual="Matroska" outcome="pass"/>
      </check>
      <check name="Video Format is FFV1">
        <context field="Format" value="FFV1"/>
        <test tracktype="Video" trackid="1" actual="FFV1" outcome="pass"/>
      </check>
      <check name="Video CodecID equals FFV1">
        <context field="CodecID" value="V_MS/VFW/FOURCC / FFV1"/>
        <test tracktype="Video" trackid="1" actual="V_MS/VFW/FOURCC / FFV1" outcome="pass"/>
      </check>
      <check name="Video Width equals 720 (pixels)">
        <context field="Width" value="720"/>
        <test tracktype="Video" trackid="1" actual="720" outcome="pass"/>
      </check>
      <check name="Video Height equals 486 (pixels)">
        <context field="Height" value="486"/>
        <test tracktype="Video" trackid="1" actual="540" outcome="fail" reason="is not equal"/>
      </check>
      <check name="Video DisplayAspectRatio equals 4:3 (1.333)">
        <context field="DisplayAspectRatio" value="1.333"/>
        <test tracktype="Video" trackid="1" actual="1.333" outcome="pass"/>
      </check>
      <check name="Video FrameRate equals 29.97 fps (29.970)">
        <context field="FrameRate" value="29.970"/>
        <test tracktype="Video" trackid="1" actual="29.970" outcome="pass"/>
      </check>
      <check name="Video Standard equals NTSC">
        <context field="Standard" value="NTSC"/>
      </check>
      <check name="Video ColorSpace equals YUV">
        <context field="ColorSpace" value="YUV"/>
        <test tracktype="Video" trackid="1" actual="YUV" outcome="pass"/>
      </check>
      <check name="Video ChromaSubsampling equals 4:2:0">
        <context field="ChromaSubsampling" value="4:2:0"/>
        <test tracktype="Video" trackid="1" actual="4:4:4" outcome="fail" reason="is not equal"/>
      </check>
      <check name="Video BitDepth equals 8 (bits)">
        <context field="BitDepth" value="8"/>
        <test tracktype="Video" trackid="1" actual="8" outcome="pass"/>
      </check>
      <check name="Audio Format equals PCM">
        <context field="Format" value="PCM"/>
        <test tracktype="Audio" trackid="2" actual="PCM" outcome="pass"/>
      </check>
      <check name="Audio Channels are greater or equal than 1">
        <context field="Channels" value="1"/>
        <test tracktype="Audio" trackid="2" actual="2" outcome="pass"/>
      </check>
      <check name="Audio SamplingRate is greater or equal than 48 kHz (48000)">
        <context field="SamplingRate" value="48000"/>
        <test tracktype="Audio" trackid="2" actual="48000" outcome="pass"/>
      </check>
      <check name="Audio BitDepth is greater or equal than 16-bit">
        <context field="BitDepth" value="16"/>
        <test tracktype="Audio" trackid="2" actual="16" outcome="pass"/>
      </check>
    </policyChecks>
  </media>
</MediaConch>
""".strip()  # noqa: E501
XML_FAILED_CHECK_OUTPUT = """
<MediaConch xmlns="https://mediaarea.net/mediaconch" xmlns:mmt="https://mediaarea.net/micromediatrace" xmlns:mi="https://mediaarea.net/mediainfo" version="0.3">
  <media ref="/fails.mkv">
    <policy name="NYULib_MKVFFV1_MODIFIED" type="and" rules_run="13" fail_count="2" pass_count="11" outcome="fail">
      <description>Attempt to re-implement the NYULibraries_MKVFFV1.xsl policy file of MediaConch XML v. 0.1 as a v. 0.3 .xml policy</description>
      <rule name="General Format equals Matroska" value="Format" tracktype="General" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='General'][*]/mi:Format='Matroska'" outcome="pass"/>
      <rule name="Video Format is FFV1" value="Format" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:Format='FFV1'" outcome="pass"/>
      <rule name="Video CodecID equals FFV1" value="CodecID" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:CodecID='V_MS/VFW/FOURCC / FFV1'" outcome="pass"/>
      <rule name="Video Width equals 720 (pixels)" value="Width" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:Width='720'" outcome="pass"/>
      <rule name="Video Height equals 486 (pixels)" value="Height" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:Height='486'" actual="540" outcome="fail"/>
      <rule name="Video DisplayAspectRatio equals 4:3 (1.333)" value="DisplayAspectRatio" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:DisplayAspectRatio='1.333'" outcome="pass"/>
      <rule name="Video ColorSpace equals YUV" value="ColorSpace" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:ColorSpace='YUV'" outcome="pass"/>
      <rule name="Video ChromaSubsampling equals 4:2:0" value="ChromaSubsampling" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:ChromaSubsampling='4:2:0'" actual="4:4:4" outcome="fail"/>
      <rule name="Video BitDepth equals 8 (bits)" value="BitDepth" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:BitDepth='8'" outcome="pass"/>
      <rule name="Audio Format equals PCM" value="Format" tracktype="Audio" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:Format='PCM'" outcome="pass"/>
      <rule name="Audio Channels are greater or equal than 1" value="Channels" tracktype="Audio" occurrence="*" operator="&gt;=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:Channels&gt;='1'" outcome="pass"/>
      <rule name="Audio SamplingRate is greater or equal than 48 kHz (48000)" value="SamplingRate" tracktype="Audio" occurrence="*" operator="&gt;=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:SamplingRate&gt;='48000'" outcome="pass"/>
      <rule name="Audio BitDepth is greater or equal than 16-bit" value="BitDepth" tracktype="Audio" occurrence="*" operator="&gt;=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:BitDepth&gt;='16'" outcome="pass"/>
    </policy>
  </media>
</MediaConch>
""".strip()  # noqa: E501
XSL_PASSED_CHECK_OUTPUT = """
<MediaConch xmlns="https://mediaarea.net/mediaconch" xmlns:mi="https://mediaarea.net/mediainfo" version="0.1">
  <media ref="/passes.mkv">
    <policyChecks>
      <name>Preservation Master File Recommendations - Matroska/FFV1 (NYU Libraries)</name>
      <description/>
      <check name="General Format equals Matroska">
        <context field="Format" value="Matroska"/>
        <test tracktype="General" actual="Matroska" outcome="pass"/>
      </check>
      <check name="Video Format is FFV1">
        <context field="Format" value="FFV1"/>
        <test tracktype="Video" trackid="1" actual="FFV1" outcome="pass"/>
      </check>
      <check name="Video CodecID equals FFV1">
        <context field="CodecID" value="V_MS/VFW/FOURCC / FFV1"/>
        <test tracktype="Video" trackid="1" actual="V_MS/VFW/FOURCC / FFV1" outcome="pass"/>
      </check>
      <check name="Video Width equals 720 (pixels)">
        <context field="Width" value="720"/>
        <test tracktype="Video" trackid="1" actual="720" outcome="pass"/>
      </check>
      <check name="Video Height equals 486 (pixels)">
        <context field="Height" value="486"/>
        <test tracktype="Video" trackid="1" actual="486" outcome="pass"/>
      </check>
      <check name="Video DisplayAspectRatio equals 4:3 (1.333)">
        <context field="DisplayAspectRatio" value="1.333"/>
        <test tracktype="Video" trackid="1" actual="1.333" outcome="pass"/>
      </check>
      <check name="Video FrameRate equals 29.97 fps (29.970)">
        <context field="FrameRate" value="29.970"/>
        <test tracktype="Video" trackid="1" actual="29.970" outcome="pass"/>
      </check>
      <check name="Video Standard equals NTSC">
        <context field="Standard" value="NTSC"/>
        <test tracktype="Video" trackid="1" actual="NTSC" outcome="pass"/>
      </check>
      <check name="Video ColorSpace equals YUV">
        <context field="ColorSpace" value="YUV"/>
        <test tracktype="Video" trackid="1" actual="YUV" outcome="pass"/>
      </check>
      <check name="Video ChromaSubsampling equals 4:2:0">
        <context field="ChromaSubsampling" value="4:2:0"/>
        <test tracktype="Video" trackid="1" actual="4:2:0" outcome="pass"/>
      </check>
      <check name="Video BitDepth equals 8 (bits)">
        <context field="BitDepth" value="8"/>
        <test tracktype="Video" trackid="1" actual="8" outcome="pass"/>
      </check>
      <check name="Audio Format equals PCM">
        <context field="Format" value="PCM"/>
        <test tracktype="Audio" trackid="2" actual="PCM" outcome="pass"/>
      </check>
      <check name="Audio Channels are greater or equal than 1">
        <context field="Channels" value="1"/>
        <test tracktype="Audio" trackid="2" actual="4" outcome="pass"/>
      </check>
      <check name="Audio SamplingRate is greater or equal than 48 kHz (48000)">
        <context field="SamplingRate" value="48000"/>
        <test tracktype="Audio" trackid="2" actual="48000" outcome="pass"/>
      </check>
      <check name="Audio BitDepth is greater or equal than 16-bit">
        <context field="BitDepth" value="16"/>
        <test tracktype="Audio" trackid="2" actual="16" outcome="pass"/>
      </check>
    </policyChecks>
  </media>
</MediaConch>
""".strip()  # noqa: E501
XML_PASSED_CHECK_OUTPUT = """
<MediaConch xmlns="https://mediaarea.net/mediaconch" xmlns:mmt="https://mediaarea.net/micromediatrace" xmlns:mi="https://mediaarea.net/mediainfo" version="0.3">
  <media ref="/passes.mkv">
    <policy name="NYULib_MKVFFV1_MODIFIED" type="and" rules_run="13" fail_count="0" pass_count="13" outcome="pass">
      <description>Attempt to re-implement the NYULibraries_MKVFFV1.xsl policy file of MediaConch XML v. 0.1 as a v. 0.3 .xml policy</description>
      <rule name="General Format equals Matroska" value="Format" tracktype="General" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='General'][*]/mi:Format='Matroska'" outcome="pass"/>
      <rule name="Video Format is FFV1" value="Format" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:Format='FFV1'" outcome="pass"/>
      <rule name="Video CodecID equals FFV1" value="CodecID" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:CodecID='V_MS/VFW/FOURCC / FFV1'" outcome="pass"/>
      <rule name="Video Width equals 720 (pixels)" value="Width" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:Width='720'" outcome="pass"/>
      <rule name="Video Height equals 486 (pixels)" value="Height" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:Height='486'" outcome="pass"/>
      <rule name="Video DisplayAspectRatio equals 4:3 (1.333)" value="DisplayAspectRatio" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:DisplayAspectRatio='1.333'" outcome="pass"/>
      <rule name="Video ColorSpace equals YUV" value="ColorSpace" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:ColorSpace='YUV'" outcome="pass"/>
      <rule name="Video ChromaSubsampling equals 4:2:0" value="ChromaSubsampling" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:ChromaSubsampling='4:2:0'" outcome="pass"/>
      <rule name="Video BitDepth equals 8 (bits)" value="BitDepth" tracktype="Video" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Video'][*]/mi:BitDepth='8'" outcome="pass"/>
      <rule name="Audio Format equals PCM" value="Format" tracktype="Audio" occurrence="*" operator="=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:Format='PCM'" outcome="pass"/>
      <rule name="Audio Channels are greater or equal than 1" value="Channels" tracktype="Audio" occurrence="*" operator="&gt;=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:Channels&gt;='1'" outcome="pass"/>
      <rule name="Audio SamplingRate is greater or equal than 48 kHz (48000)" value="SamplingRate" tracktype="Audio" occurrence="*" operator="&gt;=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:SamplingRate&gt;='48000'" outcome="pass"/>
      <rule name="Audio BitDepth is greater or equal than 16-bit" value="BitDepth" tracktype="Audio" occurrence="*" operator="&gt;=" xpath="mi:MediaInfo/mi:track[@type='Audio'][*]/mi:BitDepth&gt;='16'" outcome="pass"/>
    </policy>
  </media>
</MediaConch>
""".strip()  # noqa: E501


@pytest.fixture
def mediaconch_fails():
    with mock.patch(
        "subprocess.check_output",
        side_effect=[
            XSL_FAILED_CHECK_OUTPUT.encode(),
            XML_FAILED_CHECK_OUTPUT.encode(),
        ],
    ):
        yield


@pytest.fixture
def mediaconch_passes():
    with mock.patch(
        "subprocess.check_output",
        side_effect=[
            XSL_PASSED_CHECK_OUTPUT.encode(),
            XML_PASSED_CHECK_OUTPUT.encode(),
        ],
    ):
        yield


def test_check_bad_file(mediaconch_fails, capsys):
    """Expect a policy check on a failing file to return a 0 exit code and
    print to stdout a JSON object with a 'eventOutcomeInformation'
    attribute whose value is 'fail'.
    """
    policy_checker = MediaConchPolicyCheckerCommand(policy_file_path=POLICY_XSL_PATH)
    exitcode = policy_checker.check(FILE_FAILS_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "fail"
    with open(POLICY_XSL_PATH) as filei:
        assert output["policy"] == filei.read()
    policy_checker = MediaConchPolicyCheckerCommand(policy_file_path=POLICY_XML_PATH)
    exitcode = policy_checker.check(FILE_FAILS_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "fail"
    with open(POLICY_XML_PATH) as filei:
        assert output["policy"] == filei.read()


def test_check_bad_file_str_pol(mediaconch_fails, capsys):
    """Same as ``test_check_bad_file`` except that the policy is passed as
    a string.
    """
    with open(POLICY_XSL_PATH) as filei:
        policy = filei.read()
    policy_checker = MediaConchPolicyCheckerCommand(
        policy=policy, policy_file_name=POLICY_XSL_NAME
    )
    exitcode = policy_checker.check(FILE_FAILS_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "fail"
    assert output["policy"] == policy
    with open(POLICY_XML_PATH) as filei:
        policy = filei.read()
    policy_checker = MediaConchPolicyCheckerCommand(
        policy=policy, policy_file_name=POLICY_XML_NAME
    )
    exitcode = policy_checker.check(FILE_FAILS_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "fail"
    assert output["policy"] == policy


def test_check_good_file(mediaconch_passes, capsys):
    """Expect a policy check on a passing file to return a 0 exit code and
    print to stdout a JSON object with a 'eventOutcomeInformation'
    attribute whose value is 'pass'.
    """
    policy_checker = MediaConchPolicyCheckerCommand(policy_file_path=POLICY_XSL_PATH)
    exitcode = policy_checker.check(FILE_PASSES_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "pass"
    with open(POLICY_XSL_PATH) as filei:
        assert output["policy"] == filei.read()
    policy_checker = MediaConchPolicyCheckerCommand(policy_file_path=POLICY_XML_PATH)
    exitcode = policy_checker.check(FILE_PASSES_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "pass"
    with open(POLICY_XML_PATH) as filei:
        assert output["policy"] == filei.read()


def test_check_good_file_str_pol(mediaconch_passes, capsys):
    """Same as ``test_check_good_file`` except that the policy is passed as
    a string.
    """
    with open(POLICY_XSL_PATH) as filei:
        policy = filei.read()
    policy_checker = MediaConchPolicyCheckerCommand(
        policy=policy, policy_file_name=POLICY_XSL_NAME
    )
    exitcode = policy_checker.check(FILE_PASSES_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "pass"
    assert output["policy"] == policy
    with open(POLICY_XML_PATH) as filei:
        policy = filei.read()
    policy_checker = MediaConchPolicyCheckerCommand(
        policy=policy, policy_file_name=POLICY_XML_NAME
    )
    exitcode = policy_checker.check(FILE_PASSES_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.out)
    assert exitcode == 0
    assert output["eventOutcomeInformation"] == "pass"
    assert output["policy"] == policy


def test_no_policy(capsys, tmp_path):
    """Expect a 1 exit code and a fail outcome when the policy file does
    not exist.
    """
    policy_checker = MediaConchPolicyCheckerCommand(
        policy_file_path=str(tmp_path / "fake" / "policy" / "path")
    )
    exitcode = policy_checker.check(FILE_PASSES_PATH)
    captured = capsys.readouterr()
    output = json.loads(captured.err)
    assert exitcode == 1
    assert output["eventOutcomeInformation"] == "fail"
    assert (
        output["eventOutcomeDetailNote"]
        == f"There is no policy file at {tmp_path}/fake/policy/path"
    )



"""ammcpc --- Archivematica (AM) MediaConch (MC) Policy Checker (PC)

This command-line application (and python module) is a simple wrapper around
the MediaConch tool which takes a file and a MediaConch policy file as input
and prints to stdout a JSON object indicating, in a way that Archivematica
likes, whether the file passes the policy check.

Command-line usage::

    $ ammcpc <PATH_TO_FILE> <PATH_TO_POLICY>

Python usage with a policy file path::

    >>> from ammcpc import MediaConchPolicyCheckerCommand
    >>> policy_checker = MediaConchPolicyCheckerCommand(
            policy_file_path='/path/to/my-policy.xml')
    >>> exitcode = policy_checker.check('/path/to/file.mkv')

Python usage with a policy as a string::

    >>> policy_checker = MediaConchPolicyCheckerCommand(
            policy='<?xml><policy> ... </policy>',
            policy_file_name='my-policy.xml')
    >>> exitcode = policy_checker.check('/path/to/file.mkv')

System dependencies:

- MediaConch version 16.12

"""

import json
import os
import subprocess
import sys
import tempfile
import uuid
from collections import namedtuple

from lxml import etree

__version__ = "0.3.0"

SUCCESS_CODE = 0
ERROR_CODE = 1
NS = "{https://mediaarea.net/mediaconch}"


class MediaConchException(Exception):
    pass


Parse = namedtuple("Parse", "etree_el stdout")


class MediaConchPolicyCheckerCommand:
    """MC Policy Checker Command runs
    ``mediaconch -mc -fx -p <path_to_policy_xsl_file> <target>``,
    parses the returned XML, and prints out a JSON report summarizing the
    results of the policy check.

    Initialize with the path to a policy file then call ``check``::

        >>> checker = MediaConchPolicyCheckerCommand('/path/to/policy')
        >>> checker.check('/path/to/file-to-be-checked')
    """

    def __init__(self, policy_file_path=None, policy=None, policy_file_name=None):
        self._policy_file_path = policy_file_path
        self._policy = policy
        self._policy_file_name = policy_file_name
        self.policy_file_name = None

    def check(self, target):
        """Return 0 if MediaConch can successfully assess whether the file at
        `target` passes the policy checks that are relevant to it, given its
        purpose and the state of the FPR. Parse the XML output by MediaConch
        and print a JSON representation of that output.
        """
        try:
            self._validate()
            parse = self._parse_mediaconch_output(target)
            policy_checks = _get_policy_checks(parse.etree_el)
            info, detail = self._get_evt_out_inf_detail(policy_checks)
            stdout = parse.stdout.decode("utf8")
            print(
                json.dumps(
                    {
                        "eventOutcomeInformation": info,
                        "eventOutcomeDetailNote": detail,
                        "policy": self.policy,
                        "policyFileName": self.policy_file_name,
                        "stdout": stdout,
                    }
                )
            )
            return SUCCESS_CODE
        except MediaConchException as exc:
            return self._error(exc)

    def _validate(self):
        if self._policy_file_path:
            if not os.path.isfile(self._policy_file_path):
                raise MediaConchException(
                    f"There is no policy file at {self._policy_file_path}"
                )
            self.policy_file_name = os.path.basename(self._policy_file_path)
            with open(self._policy_file_path) as filei:
                self.policy = filei.read()
        elif self._policy and self._policy_file_name:
            self.policy_file_name = self._policy_file_name
            self.policy = self._policy
        else:
            raise MediaConchException(
                "You must supply the path to a MediaConch policy file or"
                " the text of the policy and its name as strings."
            )

    def _parse_mediaconch_output(self, target):
        """Run ``mediaconch -mc -fx -p <path_to_policy_xsl_file>
        <target>`` against the file at ``path_to_target`` and return an lxml
        etree parse of the output.
        """
        try:
            if self._policy_file_path:
                args = [
                    "mediaconch",
                    "-mc",
                    "-fx",
                    "-p",
                    self._policy_file_path,
                    target,
                ]
                output = subprocess.check_output(args)
            else:
                ext = os.path.splitext(self.policy_file_name)[1]
                pfp = tempfile.NamedTemporaryFile(suffix=ext, delete=False)
                try:
                    pfp.write(self._policy.encode("utf8"))
                    pfp.close()
                    args = ["mediaconch", "-mc", "-fx", "-p", pfp.name, target]
                    output = subprocess.check_output(args)
                finally:
                    if not pfp.closed:
                        pfp.close()
                    os.unlink(pfp.name)
        except subprocess.CalledProcessError:
            raise MediaConchException(
                "MediaConch failed when running: {}".format(" ".join(args))
            )
        try:
            return Parse(etree_el=etree.fromstring(output), stdout=output)
        except etree.XMLSyntaxError:
            raise MediaConchException(
                "The MediaConch command failed when attempting to parse the"
                f" XML output by MediaConch:\n\n {output}"
            )

    def _get_evt_out_inf_detail(self, policy_checks):
        """Return a 2-tuple of info and detail.
        - info: 'pass' or 'fail'
        - detail: human-readable string indicating which policy checks
        passed or failed. If the policy check as a whole passed, just return
        the passed check names; if it failed, just return the failed ones.
        """
        if policy_checks["mc_version"] == "0.3":
            return self._get_evt_out_inf_detail_v_0_3(policy_checks)
        return self._get_evt_out_inf_detail_v_0_1(policy_checks)

    def _get_evt_out_inf_detail_v_0_3(self, policy_checks):
        failed_policy_checks = set()
        passed_policy_checks = set()
        info = "fail"
        if policy_checks["root_policy"][1] == "pass":
            info = "pass"
        for name, outcome in policy_checks["policies"]:
            if outcome == "pass":
                passed_policy_checks.add(name)
            else:
                failed_policy_checks.add("failed policy/rule: %s" % name)
        prefix = (
            "MediaConch policy check result against policy file"
            f" {self.policy_file_name}:"
        )
        if info == "fail":
            return ("fail", "{} {}".format(prefix, "; ".join(failed_policy_checks)))
        if passed_policy_checks:
            return (
                "pass",
                "{} All policy checks passed: {}".format(
                    prefix, "; ".join(passed_policy_checks)
                ),
            )
        return (
            "pass",
            f"{prefix} No checks passed, but none failed" " either.",
        )

    def _get_evt_out_inf_detail_v_0_1(self, policy_checks):
        failed_policy_checks = set()
        passed_policy_checks = set()
        for name, (out, fie, act, rea) in policy_checks["policy_checks"].items():
            if out == "pass":
                passed_policy_checks.add(name)
            else:
                failed_policy_checks.add(
                    f'The check "{name}" failed; the actual value for the'
                    f' field "{fie}" was "{act}"; the reason was'
                    f' "{rea}".'
                )
        prefix = (
            "MediaConch policy check result against policy file"
            f" {self.policy_file_name}:"
        )
        if failed_policy_checks:
            return ("fail", "{} {}".format(prefix, " ".join(failed_policy_checks)))
        if passed_policy_checks:
            return (
                "pass",
                "{} All policy checks passed: {}".format(
                    prefix, "; ".join(passed_policy_checks)
                ),
            )
        return (
            "pass",
            f"{prefix} No checks passed, but none failed" " either.",
        )

    def _error(self, exc):
        try:
            pfn = self.policy_file_name
            pol = self.policy
        except AttributeError:
            pfn = pol = None
        print(
            json.dumps(
                {
                    "eventOutcomeInformation": "fail",
                    "eventOutcomeDetailNote": str(exc),
                    "policy": pol,
                    "policyFileName": pfn,
                    "stdout": None,
                }
            ),
            file=sys.stderr,
        )
        return ERROR_CODE


def _get_policy_check_name(policy_check_el):
    return policy_check_el.attrib.get("name", "Unnamed Check %s" % uuid.uuid4())


def _parse_policy_check_test(policy_check_el):
    """Return a 3-tuple parse of the <test> element of the policy <check>
    element.

    - El1 is outcome ("pass" or "fail" or other?)
    - El2 is the relevant field (i.e., attribute of the file)
    - El3 is the actual value of the relevant attribute/field.
    - El4 is the reason for the failure.
    """
    test_el = policy_check_el.find("%stest" % NS)
    if test_el is None:
        return None
    field = "no field"
    context_el = policy_check_el.find("%scontext" % NS)
    if context_el is not None:
        field = context_el.attrib.get("field", "no field")
    return (
        test_el.attrib.get("outcome", "no outcome"),
        field,
        test_el.attrib.get("actual", "no actual value"),
        test_el.attrib.get("reason", "no reason"),
    )


def _get_policy_checks_v_0_3(doc):
    policy_checks = {"mc_version": "0.3", "policies": []}
    root_policy = doc.find(f".{NS}media/{NS}policy")
    if root_policy is None:
        raise MediaConchException("Unable to find a root policy")
    policy_checks["root_policy"] = (
        root_policy.attrib.get("name", "No root policy name"),
        root_policy.attrib.get("outcome", "No root policy outcome"),
    )
    for el_tname in ("policy", "rule"):
        path = f".//{NS}{el_tname}"
        for policy_el in doc.iterfind(path):
            policy_name = _get_policy_check_name(policy_el)
            policy_checks["policies"].append(
                (policy_name, policy_el.attrib.get("outcome", "no outcome"))
            )
    return policy_checks


def _get_policy_checks_v_0_1(doc):
    policy_checks = {"mc_version": "0.1", "policy_checks": {}}
    path = f".{NS}media/{NS}policyChecks/{NS}check"
    for policy_check_el in doc.iterfind(path):
        policy_check_name = _get_policy_check_name(policy_check_el)
        parse = _parse_policy_check_test(policy_check_el)
        if parse:
            policy_checks["policy_checks"][policy_check_name] = parse
    return policy_checks


def _get_policy_checks(doc):
    """Get all of the policy check names and outcomes from the policy check
    output file parsed as ``doc``.
    """
    mc_xml_vrsn = doc.attrib.get("version", "No identifiable MediaConch XML version")
    if mc_xml_vrsn == "0.3":
        return _get_policy_checks_v_0_3(doc)
    elif mc_xml_vrsn == "0.1":
        return _get_policy_checks_v_0_1(doc)
    else:
        raise MediaConchException(
            "Unable to parse MediaConch XML files with version" f' "{mc_xml_vrsn}"'
        )


def main():
    target = sys.argv[1]
    policy = sys.argv[2]
    policy_checker = MediaConchPolicyCheckerCommand(policy_file_path=policy)
    sys.exit(policy_checker.check(target))
